#include <genesis.h>
#include "res_image.h"

#define BUTTON_REPEAT_TRIGGER_COUNT 0
#define BUTTON_REPEAT_INTERVAL 5
#define NUM_OF_ANIMATION 3

/**
 * コントローラの状態
 */
typedef struct {
    u16 current; // 押下しているキー
    u16 trigger; // 押し始めたキー
    u16 off; // 離したキー
    u16 repeat; // 押し続けているキー
    u16 repeatCount; // リピートカウント
} JoyState;

/**
 * コントローラの状態を保持するグローバル変数
 */
JoyState joyState;

/**
 * コントローラ状態の初期設定
 */
static void joyInit() {
    // 初期化(0埋め)
    memset(&joyState, 0, sizeof(JoyState));
}

/**
 * VBlankごとに1回だけボタンの情報を取得する
 */
static void joyExec() {
    u16 state = JOY_readJoypad(JOY_1);

    joyState.trigger = state & ~joyState.current;
    joyState.off = joyState.current & ~state;
    joyState.current = state;

    // 「押し続けているキー」の設定
    // NOTE: 「押し続けているキー」は、押し始めたあと指定の時間無効化され、
    //   それ以降押下している間は一定の間隔でオンオフを繰り返す
    if (joyState.trigger) {
        joyState.repeat = joyState.current;
        joyState.repeatCount = BUTTON_REPEAT_TRIGGER_COUNT;
    }
    else if (!joyState.repeatCount) {
        joyState.repeat = joyState.current;
        joyState.repeatCount = BUTTON_REPEAT_INTERVAL;
    }
    else {
        joyState.repeat = 0;
    }
    // カウント
    if (joyState.current) {
        if (joyState.repeatCount > 0) {
            joyState.repeatCount--;
        }
    }
}

int main(u16 hard) {
    // タイルのVRAM位置
    u16 index = TILE_USERINDEX;

    // コントローラ状態の初期化
    joyInit();

    // 色の設定
    PAL_setPaletteColors(0, &tire_palette, DMA);

    // スプライトエンジンの初期化
    SPR_init();

    // スプライトの追加と取得
    Sprite* tire = SPR_addSprite(&tire_sprite, 144, 88, TILE_ATTR_FULL(PAL0, FALSE, FALSE, FALSE, index));

    // 操作説明テキスト
    VDP_drawText("A Button:Next Frame", 9, 17);
    VDP_drawText("B Button:Next Animation", 9, 18);

    // メインループ
    while (TRUE) {
        SYS_doVBlankProcess();

        // ボタン状態取得
        joyExec();

        if ((joyState.trigger & BUTTON_A) || (joyState.repeat & BUTTON_A)) {
            // スプライトのフレームを1つ進める
            SPR_nextFrame(tire);
        }
        if (joyState.trigger & BUTTON_B) {
            // スプライトのアニメーションを次のセットにする
            SPR_setAnim(tire, (tire->animInd + 1) % NUM_OF_ANIMATION);
        }

        // スプライト更新(スプライトに対して行った変更が反映される)
        SPR_update();
    }

    return 0;
}